@@ -213,10 +213,10 @@ void init_malloc_pointers() {
   // released with a FreeLibrary call, as the functions used from this
   // library can be called from global object destructors late in the
   // process lifecycle. Not calling FreeLibrary is benign is this case.
-  HMODULE hlib = LoadLibraryEx(jemalloc_dll_name, NULL,
+  HMODULE hlib = LoadLibraryEx(jemalloc_dll_name, nullptr,
                                LOAD_LIBRARY_SEARCH_APPLICATION_DIR);
-  if (NULL == hlib) {
-    DWORD err = GetLastError();
+  if (nullptr == hlib) {
+    const DWORD err = GetLastError();
     if (ERROR_MOD_NOT_FOUND == err) {
       // Normal behaviour: not finding the jemalloc dll means
       // we don't want to use it.
@@ -492,7 +492,7 @@ void *my_internal_realloc(void *oldpoint, size_t size, myf my_flags) {
   assert(size > 0);
   /* These flags are mutually exclusive. */
   assert(!((my_flags & MY_FREE_ON_ERROR) && (my_flags & MY_HOLD_ON_ERROR)));
-  DBUG_EXECUTE_IF("simulate_out_of_memory", point = NULL; goto end;);
+  DBUG_EXECUTE_IF("simulate_out_of_memory", point = nullptr; goto end;);
   if (!oldpoint && (my_flags & MY_ALLOW_ZERO_PTR))
     return my_raw_malloc<allocator>(size, my_flags);
 #if defined(MY_MSCRT_DEBUG)
@@ -547,7 +547,7 @@ void *my_memdup(PSI_memory_key key, const void *from, size_t length,
 
 char *my_strdup(PSI_memory_key key, const char *from, myf my_flags) {
   char *ptr;
-  size_t length = strlen(from) + 1;
+  const size_t length = strlen(from) + 1;
   if ((ptr = (char *)my_malloc(key, length, my_flags)))
     memcpy(ptr, from, length);
   return ptr;
@@ -563,10 +563,8 @@ char *my_strndup(PSI_memory_key key, const char *from, size_t length,
   return ptr;
 }
 
-void my_memset_s(void *dest, size_t dest_max, int c, size_t n) {
-#if defined(HAVE_MEMSET_S)
-  memset_s(dest, dest_max, c, n);
-#else
+#if !defined(HAVE_MEMSET_S)
+void memset_s(void *dest, size_t dest_max, int c, size_t n) {
 #if defined(WIN32)
   SecureZeroMemory(dest, n);
 #else
@@ -575,5 +573,5 @@ void my_memset_s(void *dest, size_t dest_max, int c, size_t n) {
     *p++ = c;
   }
 #endif
-#endif
 }
+#endif
