@@ -21,12 +21,6 @@ struct gcs_act
         buf_len(bl),
         type(t)
     { }
-    gcs_act(const gcs_act& a)
-        :
-        buf(a.buf),
-        buf_len(a.buf_len),
-        type(a.type)
-    { }
 };
 
 struct gcs_act_rcvd
@@ -36,8 +30,8 @@ struct gcs_act_rcvd
     gcs_seqno_t    id;          // global total order seqno
     int            sender_idx;
     char           sender_id[GU_UUID_STR_LEN + 1];
-    gcs_act_rcvd() : act(), local(NULL), id(GCS_SEQNO_ILL), sender_idx(-1) { 
-      sender_id[0] = 0;
+    gcs_act_rcvd() : act(), local(NULL), id(GCS_SEQNO_ILL), sender_idx(-1) {
+        sender_id[0] = 0;
     }
     gcs_act_rcvd(const gcs_act& a, const struct gu_buf* loc,
                  gcs_seqno_t i, int si)
@@ -47,7 +41,7 @@ struct gcs_act_rcvd
         id(i),
         sender_idx(si)
     {
-      sender_id[0] = 0;
+        sender_id[0] = 0;
     }
 };
 
