@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -57,6 +57,7 @@
 #include "xcom/xcom_detector.h"
 #include "xcom/xcom_memory.h"
 #include "xcom/xcom_msg_queue.h"
+#include "xcom/xcom_scope_guard.h"
 #include "xcom/xcom_statistics.h"
 #include "xcom/xcom_transport.h"
 #include "xcom/xcom_vp_str.h"
@@ -131,6 +132,18 @@ connection_descriptor *open_new_local_connection(const char *server,
 }
 /* purecov: end */
 
+bool is_able_to_connect_to_node(const char *server, const xcom_port port) {
+  connection_descriptor *new_conn{nullptr};
+
+  Xcom_scope_guard cleanup_guard([&]() { free(new_conn); });
+
+  if (new_conn = open_new_connection(server, port, 1000); new_conn->fd == -1) {
+    return false;
+  }
+
+  return !static_cast<bool>(close_open_connection(new_conn));
+}
+
 result set_nodelay(int fd) {
   int n = 1;
   result ret = {0, 0};
@@ -419,6 +432,7 @@ int send_proto(connection_descriptor *con, xcom_proto x_proto,
 #define XDRFUNC xdrfunc
 #endif
 
+// This function is annotated with SUPPRESS_UBSAN, see header file declaration.
 int apply_xdr(void *buff, uint32_t bufflen, xdrproc_t xdrfunc, void *xdrdata,
               enum xdr_op op) {
   XDR xdr;
@@ -520,7 +534,7 @@ static inline int old_proto_knows(xcom_proto x_proto [[maybe_unused]],
 }
 
 static xdrproc_t pax_msg_func[] = {
-    reinterpret_cast<xdrproc_t>(0),
+    nullptr,
     reinterpret_cast<xdrproc_t>(xdr_pax_msg_1_0),
     reinterpret_cast<xdrproc_t>(xdr_pax_msg_1_1),
     reinterpret_cast<xdrproc_t>(xdr_pax_msg_1_2),
