@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -28,8 +28,8 @@
 #include <string.h>
 #include <sys/types.h>
 
-#include "m_ctype.h"
 #include "my_inttypes.h"
+#include "mysql/strings/m_ctype.h"
 #include "sql/dd/impl/object_key.h"  // dd::Object_key
 #include "sql/dd/object_id.h"        // dd::Object_id
 #include "sql/dd/string_type.h"
@@ -247,10 +247,9 @@ class Se_private_id_key : public Object_key {
   String_type str() const override;
 
   bool operator<(const Se_private_id_key &rhs) const {
-    return m_private_id < rhs.m_private_id
-               ? true
-               : rhs.m_private_id < m_private_id ? false
-                                                 : m_engine < rhs.m_engine;
+    return m_private_id < rhs.m_private_id   ? true
+           : rhs.m_private_id < m_private_id ? false
+                                             : m_engine < rhs.m_engine;
   }
 
  private:
