@@ -61,12 +61,6 @@ struct Error_handler {
 DEFINE_BOOL_METHOD(mysql_command_services_imp::init, (MYSQL_H * mysql_h)) {
   bool ret = false;
   try {
-    THD *thd = current_thd;
-    /* Not supposed to work recursively */
-    if (thd != nullptr && thd->is_a_srv_session()) {
-      my_error(ER_CANT_RUN_COMMAND_SERVICES_RECURSIVELY, MYF(0));
-      return true;
-    }
     Mysql_handle *mysql_handle = (Mysql_handle *)my_malloc(
         key_memory_query_service, sizeof(Mysql_handle),
         MYF(MY_WME | MY_ZEROFILL));
@@ -913,7 +907,7 @@ DEFINE_METHOD(int, mysql_command_services_imp::next_result, (MYSQL_H mysql_h)) {
     Mysql_handle *m_handle = reinterpret_cast<Mysql_handle *>(mysql_h);
     if (m_handle == nullptr) return true;
     /* more results? -1 = no, >0 = error, 0 = yes (keep looping) */
-    int ret = mysql_next_result(m_handle->mysql);
+    const int ret = mysql_next_result(m_handle->mysql);
     return ret;
   } catch (...) {
     mysql_components_handle_std_exception(__func__);
