@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2024, Oracle and/or its affiliates.
+Copyright (c) 1996, 2025, Oracle and/or its affiliates.
 Copyright (c) 2008, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -230,7 +230,7 @@ void btr_search_sys_resize(ulint hash_size) {
     ib::error(ER_IB_MSG_45) << "btr_search_sys_resize failed because"
                                " hash index hash table is not empty.";
     ut_d(ut_error);
-    ut_o(return );
+    ut_o(return);
   }
 
   /* Step-2: Recreate hash tables with new size. */
@@ -869,7 +869,7 @@ bool btr_search_guess_on_hash(const dtuple_t *tuple, ulint mode,
     }
   } else {
     /* If we had a latch, then the guard is not needed. */
-    latch_guard.commit();
+    latch_guard.release();
   }
 
   ut_ad(rw_lock_get_writer(btr_get_search_latch(index)) != RW_LOCK_X);
@@ -906,7 +906,7 @@ bool btr_search_guess_on_hash(const dtuple_t *tuple, ulint mode,
     remove it. Up to this point we have the AHI is S-latched and since we found
     an AHI entry that leads to this block, the entry can't be removed and thus
     the block must be still in the buffer pool. */
-    latch_guard.rollback();
+    latch_guard.reset();
 
     buf_block_dbg_add_level(block, SYNC_TREE_NODE_FROM_HASH);
   }
@@ -1578,7 +1578,7 @@ static void btr_search_build_page_hash_index(dict_index_t *index,
     ha_insert_for_hash(table, hashes[i], block, recs[i]);
   }
 
-  x_latch_guard.rollback();
+  x_latch_guard.reset();
 
   MONITOR_ATOMIC_INC(MONITOR_ADAPTIVE_HASH_PAGE_ADDED);
 }
