@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 2018, 2024, Oracle and/or its affiliates.
+Copyright (c) 2018, 2025, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -637,6 +637,12 @@ void Clone_persist_gtid::periodic_write() {
     os_event_reset(m_event);
     /* Write accumulated GTIDs to disk table */
     flush_gtids(thd);
+    /* Memory is allocated from thd->memroot to store the information about the
+    table being locked for flushing gtids. In this case, being a background
+    thread which is flushing the gtids every 100 millisecond, we should release
+    the memory after every cycle of flushing gtids to avoid memory growth in
+    Clone_persist_gtid thread .*/
+    thd->mem_root->ClearForReuse();
   }
 
   /* For slow shutdown, consume remaining GTIDs so that undo can be purged. */
