@@ -474,7 +474,7 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
     byte *data;
     ulint longest = 0;
     ulint longest_i = ULINT_MAX;
-    const upd_field_t *uf = nullptr;
+    upd_field_t *uf = nullptr;
 
     for (ulint i = dict_index_get_n_unique_in_tree(index);
          i < dtuple_get_n_fields(entry); i++) {
@@ -606,8 +606,7 @@ big_rec_t *dtuple_convert_big_rec(dict_index_t *index, upd_t *upd,
     if (upd == nullptr) {
       big_rec.ext_in_old = false;
     } else {
-      const upd_field_t *const uf =
-          upd->get_field_by_field_no(longest_i, index);
+      upd_field_t *uf = upd->get_field_by_field_no(longest_i, index);
       ut_ad(uf != nullptr);
       big_rec.ext_in_old = uf->ext_in_old;
     }
@@ -658,7 +657,7 @@ big_rec_t *big_rec_t::alloc(mem_heap_t *heap, ulint n_fld) {
       mem_heap_alloc(heap, n_fld * sizeof(big_rec_field_t)));
 
   rec->n_fields = 0;
-  return rec;
+  return (rec);
 }
 
 dfield_t *dfield_t::clone(mem_heap_t *heap) {
@@ -678,13 +677,13 @@ dfield_t *dfield_t::clone(mem_heap_t *heap) {
     obj->data = nullptr;
   }
 
-  return obj;
+  return (obj);
 }
 
 byte *dfield_t::blobref() const {
   ut_ad(ext);
 
-  return static_cast<byte *>(data) + len - BTR_EXTERN_FIELD_REF_SIZE;
+  return (static_cast<byte *>(data) + len - BTR_EXTERN_FIELD_REF_SIZE);
 }
 
 uint32_t dfield_t::lob_version() const {
@@ -862,6 +861,9 @@ bool multi_value_data::has(ulint mtype, ulint prtype, const byte *data,
   return (false);
 }
 
+void dtuple_t::set_min_rec_flag() { info_bits |= REC_INFO_MIN_REC_FLAG; }
+void dtuple_t::unset_min_rec_flag() { info_bits &= ~REC_INFO_MIN_REC_FLAG; }
+
 #endif /* !UNIV_HOTBACKUP */
 
 void multi_value_data::alloc(uint32_t num, bool alc_bitset, mem_heap_t *heap) {
@@ -970,9 +972,9 @@ byte *Multi_value_logger::log(byte **ptr) {
     rather than the size of bitset. */
     uint32_t bitset_len = UT_BITS_IN_BYTES(m_mv_data->num_v);
     /* make sure we will not read past the buffer */
-    ut_ad(bitset_len <= m_mv_data->bitmap_bytes);
+    ut_ad_le(bitset_len, m_mv_data->bitmap_bytes);
     /* make sure we will not miss actual data */
-    ut_ad(m_mv_data->bitset->size_bytes() <= bitset_len);
+    ut_ad_le(m_mv_data->bitset->size_bytes(), bitset_len);
     ut_memcpy(*ptr, m_mv_data->bitset->data(), bitset_len);
     *ptr += bitset_len;
   }
@@ -1033,9 +1035,9 @@ const byte *Multi_value_logger::read(const byte *ptr, dfield_t *field,
 
   if (ptr < old_ptr + total_len) {
     multi_val->alloc_bitset(heap);
-    ut_ad(num == multi_val->num_v);
-    ut_ad(UT_BITS_IN_BYTES(num) == multi_val->bitmap_bytes);
-    ut_a(UT_BITS_IN_BYTES(num) == multi_val->bitset->size_bytes());
+    ut_ad_eq(num, multi_val->num_v);
+    ut_ad_eq(UT_BITS_IN_BYTES(num), multi_val->bitmap_bytes);
+    ut_a_eq(UT_BITS_IN_BYTES(num), multi_val->bitset->size_bytes());
     multi_val->bitset->copy_from(ptr);
     ptr += UT_BITS_IN_BYTES(num);
   }
@@ -1070,3 +1072,11 @@ uint32_t Multi_value_logger::get_keys_capacity(uint32_t log_size,
 
   return (keys_length);
 }
+
+dtuple_t *dtuple_t::deep_copy(mem_heap_t *heap) const {
+  dtuple_t *copy = dtuple_copy(this, heap);
+  for (uint32_t i = 0; i < n_fields; ++i) {
+    dfield_dup(&copy->fields[i], heap);
+  }
+  return copy;
+}
