@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2024, Oracle and/or its affiliates.
+Copyright (c) 1996, 2025, Oracle and/or its affiliates.
 Copyright (c) 2012, Facebook Inc.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -290,6 +290,10 @@ in constraints which reference it */
 be accessed by the caller afterwards */
 void dict_index_remove_from_cache(dict_table_t *table, dict_index_t *index);
 
+/** Gathers ids of all tables in cache at the moment.
+@return ids of all tables */
+std::vector<table_id_t> dict_get_all_table_ids();
+
 /** Change the id of a table object in the dictionary cache. This is used in
  DISCARD TABLESPACE. */
 void dict_table_change_id_in_cache(
@@ -1642,27 +1646,10 @@ static inline bool dict_table_is_partition(const dict_table_t *table);
 @param[in]      index   index object */
 static inline void dict_allocate_mem_intrinsic_cache(dict_index_t *index);
 
-/** Evict all tables that are loaded for applying purge.
-Since we move the offset of all table ids during upgrade,
-these tables cannot exist in cache. Also change table_ids
-of SYS_* tables if they are upgraded from earlier versions */
-void dict_upgrade_evict_tables_cache();
-
 /** @return true if table is InnoDB SYS_* table
 @param[in]      table_id        table id  */
 bool dict_table_is_system(table_id_t table_id);
 
-/** Build the table_id array of SYS_* tables. This
-array is used to determine if a table is InnoDB SYSTEM
-table or not.
-@return true if successful, false otherwise */
-bool dict_sys_table_id_build();
-
-/** Change the table_id of SYS_* tables if they have been created after
-an earlier upgrade. This will update the table_id by adding DICT_MAX_DD_TABLES
-*/
-void dict_table_change_id_sys_tables();
-
 /** Get the tablespace data directory if set, otherwise empty string.
 @return the data directory */
 [[nodiscard]] std::string dict_table_get_datadir(const dict_table_t *table);
@@ -1682,7 +1669,7 @@ extern bool dict_upgrade_zip_dict_missing;
 @param[out]	dict_id		zip_dict id
 @retval	DB_SUCCESS		if OK
 @retval	DB_RECORD_NOT_FOUND	if not found */
-MY_NODISCARD
+[[nodiscard]]
 dberr_t dict_get_dictionary_id_by_key(table_id_t table_id, ulint column_pos,
                                       ulint *dict_id);
 
@@ -1696,7 +1683,7 @@ Must be freed with mem_free().
 @param[out]	data_len	dictionary data length
 @retval	DB_SUCCESS		if OK
 @retval	DB_RECORD_NOT_FOUND	if not found */
-MY_NODISCARD
+[[nodiscard]]
 dberr_t dict_get_dictionary_info_by_id(ulint dict_id, char **name,
                                        ulint *name_len, char **data,
                                        ulint *data_len);
@@ -1707,13 +1694,10 @@ This can happen if Percona Server is bootstrapped with
 upgrade, mysql.ibd should be encrpted.
 In no upgrade scenario it fetches encryption flag from first page of mysql.ibd
 to check whether it is encrypted.
-@param[in]  dict_init_mode      initalization mode
-@param[in]  mysql_plugin_space  space_id of mysql/plugin table. Used only
-                                during upgrade
-@param[out] do_encrypt          true if encrypted, false if not encrypted
+@param[in]  dict_init_mode  initalization mode
+@param[out] encrypt_mysql   true if encrypted, false if not encrypted
 @return true if success, false if failure */
 bool dict_detect_encryption_of_mysql_ibd(dict_init_mode_t dict_init_mode,
-                                         space_id_t mysql_plugin_space,
                                          bool &encrypt_mysql);
 
 /** Set the compression type for the tablespace of a table
