@@ -242,7 +242,7 @@ introduced where a call to log_free_check() is bypassed. */
     after it had been completely inserted. Therefore, we
     are passing rollback=false, just like purge does. */
 
-    btr_cur_pessimistic_delete(&err, false, btr_cur, 0, false, node->trx->id,
+    btr_cur_pessimistic_delete(&err, false, btr_cur, 0, false, node->trx.id,
                                node->undo_no, node->rec_type, mtr, &node->pcur,
                                nullptr);
 
@@ -267,8 +267,8 @@ introduced where a call to log_free_check() is bypassed. */
   dict_index_t *index;
   bool online;
 
-  ut_ad(thr_get_trx(thr) == node->trx);
-  ut_ad(node->trx->in_rollback);
+  ut_ad(thr_get_trx(thr) == &node->trx);
+  ut_ad(node->trx.in_rollback);
 
   log_free_check();
   pcur = &node->pcur;
@@ -521,7 +521,7 @@ introduced where a call to log_free_check() is bypassed. */
       the distinction only matters when deleting a
       record that contains externally stored columns. */
       ut_ad(!index->is_clustered());
-      btr_cur_pessimistic_delete(&err, false, btr_cur, 0, false, node->trx->id,
+      btr_cur_pessimistic_delete(&err, false, btr_cur, 0, false, node->trx.id,
                                  node->undo_no, node->rec_type, &mtr,
                                  &node->pcur, nullptr);
       /* The delete operation may fail if we have little
@@ -1203,7 +1203,6 @@ static dberr_t row_undo_mod_upd_exist_multi_sec(undo_node_t *node,
 static void row_undo_mod_parse_undo_rec(undo_node_t *node, THD *thd,
                                         MDL_ticket **mdl) {
   dict_index_t *clust_index;
-  byte *ptr;
   undo_no_t undo_no;
   table_id_t table_id;
   trx_id_t trx_id;
@@ -1214,8 +1213,9 @@ static void row_undo_mod_parse_undo_rec(undo_node_t *node, THD *thd,
   bool dummy_extern;
   type_cmpl_t type_cmpl;
 
-  ptr = trx_undo_rec_get_pars(node->undo_rec, &type, &cmpl_info, &dummy_extern,
-                              &undo_no, &table_id, type_cmpl);
+  auto ptr =
+      trx_undo_rec_get_pars(node->undo_rec, &type, &cmpl_info, &dummy_extern,
+                            &undo_no, &table_id, type_cmpl);
   node->rec_type = type;
 
   /* Although table IX lock is held now, DROP TABLE could still be
@@ -1279,10 +1279,10 @@ dberr_t row_undo_mod(undo_node_t *node, /*!< in: row undo node */
   ut_ad(node != nullptr);
   ut_ad(thr != nullptr);
   ut_ad(node->state == UNDO_NODE_MODIFY);
-  ut_ad(node->trx->in_rollback);
+  ut_ad(node->trx.in_rollback);
   ut_ad(!trx_undo_roll_ptr_is_insert(node->roll_ptr));
 
-  ut_ad(thr_get_trx(thr) == node->trx);
+  ut_ad(thr_get_trx(thr) == &node->trx);
 
   THD *thd = dd_thd_for_undo(node->trx);
 
