@@ -27,6 +27,7 @@
 #include <ndb_opts.h>
 #include <portlib/ndb_daemon.h>
 #include <kernel/NodeBitmask.hpp>
+#include "portlib/ssl_applink.h"
 #include "util/ndb_openssl_evp.h"
 
 #include "angel.hpp"
@@ -76,6 +77,8 @@ static struct my_option my_long_options[] = {
     NdbStdOpt::ndb_nodeid,
     NdbStdOpt::connect_retry_delay,  // used
     NdbStdOpt::connect_retries,      // used
+    NdbStdOpt::tls_search_path,
+    NdbStdOpt::mgm_tls,
     NDB_STD_OPT_DEBUG{"core-file", NDB_OPT_NOSHORT, "Write core on errors.",
                       &opt_core, nullptr, nullptr, GET_BOOL, NO_ARG,
                       OPT_WANT_CORE_DEFAULT, 0, 0, nullptr, 0, nullptr},
@@ -240,7 +243,8 @@ int real_main(int argc, char **argv) {
     ndbd_run(opt_foreground, opt_report_fd, opt_ndb_connectstring,
              opt_ndb_nodeid, opt_bind_address, opt_no_start, opt_initial,
              opt_initialstart, opt_allocated_nodeid, opt_connect_retries,
-             opt_connect_retry_delay, opt_logbuffer_size);
+             opt_connect_retry_delay, opt_logbuffer_size, opt_tls_search_path,
+             opt_mgm_tls);
   }
 
   /**
@@ -252,7 +256,8 @@ int real_main(int argc, char **argv) {
   */
   angel_run(progname, original_args, opt_ndb_connectstring, opt_ndb_nodeid,
             opt_bind_address, opt_initial, opt_no_start, opt_daemon,
-            opt_connect_retries, opt_connect_retry_delay);
+            opt_connect_retries, opt_connect_retry_delay, opt_tls_search_path,
+            opt_mgm_tls);
 
   return 1;  // Never reached
 }
