@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2024, Oracle and/or its affiliates.
+   Copyright (c) 2003, 2025, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -36,6 +36,7 @@
 #include <Vector.hpp>
 #include "my_getopt.h"
 #include "portlib/NdbTick.h"
+#include "portlib/ssl_applink.h"
 #include "util/ndb_openssl_evp.h"  // ndb_openssl_evp::library_init()
 #include "util/require.h"
 
@@ -43,6 +44,7 @@
 #include "consumer_printer.hpp"
 #include "consumer_restore.hpp"
 #include "my_alloc.h"
+#include "nulls.h"
 
 #include <NdbThread.h>
 
@@ -269,6 +271,8 @@ static struct my_option my_long_options[] = {
     NdbStdOpt::ndb_nodeid,
     NdbStdOpt::connect_retry_delay,
     NdbStdOpt::connect_retries,
+    NdbStdOpt::tls_search_path,
+    NdbStdOpt::mgm_tls,
     NDB_STD_OPT_DEBUG{
         "timestamp_printouts", NDB_OPT_NOSHORT,
         "Add a timestamp to the logger messages info, error and debug",
@@ -1181,6 +1185,16 @@ static inline bool checkDbAndTableName(const TableS *table) {
   return false;
 }
 
+static inline bool rebuildSysTableIdx(const TableS *table) {
+  const char *table_name = table->getTableName();
+  bool res = false;
+
+  res |= opt_include_stored_grants &&
+         strcmp(table_name, NDB_REP_DB "/def/" NDB_SQL_METADATA_TABLE) == 0;
+  res |= checkSysTable(table) && checkDbAndTableName(table);
+  return res;
+}
+
 static void exclude_missing_tables(const RestoreMetaData &metaData,
                                    const Vector<BackupConsumer *> g_consumers) {
   Uint32 i, j;
@@ -1622,6 +1636,7 @@ static void init_restore() {
       exitHandler(NdbToolsProgramExitCode::FAILED);
     }
     g_cluster_connection->set_name(g_options.c_str());
+    g_cluster_connection->configure_tls(opt_tls_search_path, opt_mgm_tls);
     if (g_cluster_connection->connect(opt_connect_retries - 1,
                                       opt_connect_retry_delay, 1) != 0) {
       err << "Could not connect to cluster: '"
@@ -2410,7 +2425,7 @@ int do_restore(RestoreThreadData *thrdata) {
 
     for (i = 0; i < metaData.getNoOfTables(); i++) {
       const TableS *table = metaData[i];
-      if (!(checkSysTable(table) && checkDbAndTableName(table))) continue;
+      if (!rebuildSysTableIdx(table)) continue;
       if (isBlobTable(table) || isIndex(table)) continue;
       for (Uint32 j = 0; j < g_consumers.size(); j++) {
         if (!g_consumers[j]->rebuild_indexes(*table)) {
