@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018-2021 Codership Oy <info@codership.com>
+ * Copyright (C) 2018-2025 Codership Oy <info@codership.com>
  *
  * This file is part of wsrep-lib.
  *
@@ -28,6 +28,7 @@
 #include "wsrep/thread_service.hpp"
 #include "wsrep/tls_service.hpp"
 #include "wsrep/allowlist_service.hpp"
+#include "wsrep/connection_monitor_service.hpp"
 
 #include "service_helpers.hpp"
 #include "thread_service_v1.hpp"
@@ -36,6 +37,7 @@
 #include "event_service_v1.hpp"
 #include "v26/wsrep_api.h"
 #include "v26/wsrep_node_isolation.h"
+#include "connection_monitor_service_v1.hpp"
 
 #include <dlfcn.h>
 #include <cassert>
@@ -738,6 +740,23 @@ namespace
     }
 
     wsrep_node_isolation_mode_set_fn_v1 node_isolation_mode_set;
+    wsrep_certify_fn_v1 certify_v1;
+
+    static int init_connection_monitor_service(void* dlh,
+                                      wsrep::connection_monitor_service* connection_monitor_service)
+    {
+        assert(connection_monitor_service);
+        if (not wsrep::connection_monitor_service_v1_probe(dlh))
+        {
+            return wsrep::connection_monitor_service_v1_init(dlh, connection_monitor_service);
+        }
+        return 1;
+    }
+
+    static void deinit_connection_monitor_service(void* dlh)
+    {
+        wsrep::connection_monitor_service_v1_deinit(dlh);
+    }
 }
 
 
@@ -785,6 +804,22 @@ void wsrep::wsrep_provider_v26::init_services(
     node_isolation_mode_set
         = wsrep_impl::resolve_function<wsrep_node_isolation_mode_set_fn_v1>(
             wsrep_->dlh, WSREP_NODE_ISOLATION_MODE_SET_V1);
+
+    certify_v1 = wsrep_impl::resolve_function<wsrep_certify_fn_v1>(
+        wsrep_->dlh, WSREP_CERTIFY_V1);
+
+    if (services.connection_monitor_service)
+    {
+        if (init_connection_monitor_service(wsrep_->dlh, services.connection_monitor_service))
+        {
+            wsrep::log_info() << "Provider does not support connection monitor service";
+            // provider does not support connection monitoring
+        }
+        else
+        {
+            services_enabled_.connection_monitor_service = services.connection_monitor_service;
+        }
+    }
 }
 
 void wsrep::wsrep_provider_v26::deinit_services()
@@ -798,6 +833,8 @@ void wsrep::wsrep_provider_v26::deinit_services()
     if (services_enabled_.allowlist_service)
         deinit_allowlist_service(wsrep_->dlh);
     node_isolation_mode_set = nullptr;
+    if (services_enabled_.connection_monitor_service)
+        deinit_connection_monitor_service(wsrep_->dlh);
 }
 
 wsrep::wsrep_provider_v26::wsrep_provider_v26(
@@ -985,20 +1022,31 @@ enum wsrep::provider::status
 wsrep::wsrep_provider_v26::certify(wsrep::client_id client_id,
                                    wsrep::ws_handle& ws_handle,
                                    int flags,
-                                   wsrep::ws_meta& ws_meta)
+                                   wsrep::ws_meta& ws_meta,
+                                   const seq_cb_t* seq_cb)
 {
     mutable_ws_handle mwsh(ws_handle);
     mutable_ws_meta mmeta(ws_meta, flags);
-    return map_return_value(
-        wsrep_->certify(wsrep_, client_id.get(), mwsh.native(),
-                        mmeta.native_flags(),
-                        mmeta.native()));
+    if (seq_cb && certify_v1)
+    {
+        wsrep_seq_cb_t wseq_cb{seq_cb->ctx, seq_cb->fn};
+        return map_return_value(certify_v1(wsrep_, client_id.get(),
+                                           mwsh.native(), mmeta.native_flags(),
+                                           mmeta.native(), &wseq_cb));
+    }
+    else
+    {
+        return map_return_value(
+            wsrep_->certify(wsrep_, client_id.get(), mwsh.native(),
+                            mmeta.native_flags(), mmeta.native()));
+    }
 }
 
 enum wsrep::provider::status
 wsrep::wsrep_provider_v26::bf_abort(
     wsrep::seqno bf_seqno,
     wsrep::transaction_id victim_id,
+    wsrep::client_service& /* Ignored here */,
     wsrep::seqno& victim_seqno)
 {
     wsrep_seqno_t wsrep_victim_seqno;
@@ -1096,6 +1144,7 @@ wsrep::wsrep_provider_v26::enter_toi(
 
 enum wsrep::provider::status
 wsrep::wsrep_provider_v26::leave_toi(wsrep::client_id client_id,
+                                     const wsrep::ws_meta&,
                                      const wsrep::mutable_buffer& err)
 {
     const wsrep_buf_t err_buf = { err.data(), err.size() };
